use std::collections::HashMap;
use colored::Colorize;
use serde_json::to_value;

use crate::{game::player::move_player, structs::save::SaveData, tui::{text::add_text}, utils::{json::save_json, maps::{get_current_map, get_map_events, get_npc, load_new_map}, os::move_cursor}};

lazy_static::lazy_static! {
    static ref COMMAND_SYNONYMS: HashMap<&'static str, &'static str> = {
        let mut map = HashMap::new();
        map.insert("go", "go");
        map.insert("towards", "go");
        map.insert("head", "go");
        map.insert("walk", "go");
        map.insert("move", "go");
        map.insert("mv", "go");
        map.insert("examine", "examine");
        map.insert("inspect", "examine");
        map.insert("look", "examine");
        map.insert("take", "take");
        map.insert("grab", "take");
        map.insert("pick", "take");
        map.insert("enter", "enter");
        map.insert("exit", "exit");
        map.insert("leave", "exit");
        map.insert("talk", "talk");
        map.insert("describe", "describe");
        map.insert("desc", "describe");
        map.insert("talk", "talk");
        map.insert("say", "say");
        map.insert("save", "save");
        map
    };

    static ref DIRECTIONS: HashMap<&'static str, (i8, i8)> = {
        let mut map = HashMap::new();
        map.insert("north", (0, -1));
        map.insert("n", (0, -1));
        map.insert("up", (0, -1));
        map.insert("u", (0, -1));
        map.insert("south", (0, 1));
        map.insert("s", (0, 1));
        map.insert("down", (0, 1));
        map.insert("d", (0, 1));
        map.insert("east", (1, 0));
        map.insert("e", (1, 0));
        map.insert("right", (1, 0));
        map.insert("r", (1, 0));
        map.insert("west", (-1, 0));
        map.insert("w", (-1, 0));
        map.insert("left", (-1, 0));
        map.insert("l", (-1, 0));
        map
    };

    static ref LOCATIONS: HashMap<&'static str, &'static str> = {
        let mut map = HashMap::new();
        map.insert("home", "home");
        map.insert("house", "home");
        map.insert("cave", "cave");
        map.insert("forest", "forest");
        map.insert("village", "village");
        map.insert("town", "town");
        map.insert("city", "city");
        map
    };
}

pub fn parse_command(input: &str) -> Option<(&str, String)> {
    let parts: Vec<&str> = input.trim().split_whitespace().collect();
    if parts.len() < 2 {
        return None;
    }
    
    let command = parts[0];
    let argument = parts[1..].join(" ");
    let height = if let Some((_, terminal_size::Height(h))) = terminal_size::terminal_size() {
        h
    } else {
        24 // Default height if terminal size cannot be determined
    };
    if let Some(synonym) = COMMAND_SYNONYMS.get(command) {
        // print!("{}", input.green());
        add_text(format!("> {} {}", synonym, argument).as_str());
        move_cursor(2, height - 2);
        match synonym {
            &"go" => {
                if let Some(direction) = DIRECTIONS.get(argument.as_str()) {
                    print!("{}", input.green());
                    move_player(direction.0, direction.1, false);
                } else {
                    println!("{}", format!("{} | unknown direction: {}", input, argument).red());
                }
            }
            &"enter" => {
                let events = get_map_events(get_current_map().unwrap().id.as_str())?;
                let all_coords = crate::PLAYER_COORDS.lock().unwrap().clone();
                let player_coords = all_coords.get(get_current_map().unwrap().id.as_str())?;

                let evt = events.iter().find(|e| e.evt_type == "location" && e.location[0] == player_coords.0 && e.location[1] == player_coords.1);
                if evt.is_some() {
                    let evt = evt.unwrap();
                    add_text("You entered a location.");

                    load_new_map(evt.command_id.as_str());

                    move_cursor(1, 1);
                    print!("Player: {} | Map: {}", crate::PLAYER_NAME.lock().unwrap().bold().underline(), get_current_map().unwrap().name.bold().underline());

                    // *crate::CURRENT_MAP_ID.lock().unwrap() = evt.command_id.clone();
                } else {
                    add_text("! You cannot enter this location.");
                }
            }
            &"exit" => {
                let events = get_map_events(get_current_map().unwrap().id.as_str())?;
                let all_coords = crate::PLAYER_COORDS.lock().unwrap().clone();
                let player_coords = all_coords.get(get_current_map().unwrap().id.as_str())?;

                // print!("{:?}, {:?}", events, player_coords);

                let evt = events.iter().find(|e| e.evt_type == "location" && e.location[0] == player_coords.0 && e.location[1] == player_coords.1);
                if evt.is_some() {
                    let evt = evt.unwrap();
                    add_text("You exited a location.");

                    load_new_map(evt.command_id.as_str());
                    move_cursor(1, 1);
                    print!("Player: {} | Map: {}", crate::PLAYER_NAME.lock().unwrap().bold().underline(), get_current_map().unwrap().name.bold().underline());

                    // *crate::CURRENT_MAP_ID.lock().unwrap() = evt.command_id.clone();
                } else {
                    add_text("! You cannot exit from this location.");
                }
            }
            &"describe" => {
                match argument.as_str() {
                    "item" => {
                        let events = get_map_events(get_current_map().unwrap().id.as_str())?;
                        let all_coords = crate::PLAYER_COORDS.lock().unwrap().clone();
                        let player_coords = all_coords.get(get_current_map().unwrap().id.as_str())?;

                        let evt = events.iter().find(|e| e.evt_type == "item" && e.location[0] == player_coords.0 && e.location[1] == player_coords.1);
                        if evt.is_some() {
                            let evt = evt.unwrap();
                            add_text(evt.description.as_ref().unwrap_or(&"No description available.".to_string()).as_str());
                        } else {
                            add_text("! No item to describe here.");
                        }
                    }
                    "tile" => {
                        let current_tile = crate::game::player::get_current_tile().unwrap();
                        add_text(current_tile.description.as_deref().unwrap_or("No description available."));        
                    }
                    "location" => {
                        let map = get_current_map().unwrap();
                        add_text(map.description.as_str());
                    }
                    _ => {
                        print!("{}", format!("{} | unknown argument: {}", input, argument).red());
                    }
                }
            }
            &"take" => {
                let events = get_map_events(get_current_map().unwrap().id.as_str())?;
                let all_coords = crate::PLAYER_COORDS.lock().unwrap().clone();
                let player_coords = all_coords.get(get_current_map().unwrap().id.as_str())?;
                let mut flags = crate::FLAGS.lock().unwrap();
                let evt = events.iter().find(|e| e.evt_type == "item" && e.location[0] == player_coords.0 && e.location[1] == player_coords.1);

                if !evt.is_some() {
                    add_text("! Nothing to pick up here.");
                    return None;
                }

                let evt = evt.unwrap();

                if !flags.contains_key(evt.command_id.as_str()) {
                    add_text(format!("You picked up {}.", evt.command_id).as_str());

                    crate::PLAYER_INVENTORY.lock().unwrap().push(evt.command_id.clone());
                    flags.insert(evt.command_id.clone(), true);
                } else {
                    add_text("! Nothing to pick up here.");
                }
            }
            &"talk" => {
                let events = get_map_events(get_current_map().unwrap().id.as_str())?;
                let all_coords = crate::PLAYER_COORDS.lock().unwrap().clone();
                let player_coords = all_coords.get(get_current_map().unwrap().id.as_str())?;

                let evt = events.iter().find(|e| e.evt_type == "npc" && e.location[0] == player_coords.0 && e.location[1] == player_coords.1);
                
                match evt {
                    Some(e) => {
                        let npc = get_npc(&e.command_id);
                        match npc {
                            Some(n) => {
                                add_text(format!("You talked to {}.", n.name).as_str());
                                for line in n.dialogue.iter() {
                                    add_text(line);
                                }
                                add_text("#(FFFF00)Use `say <number>` to respond");
                            }
                            None => {
                                add_text("! Strange NPC here, no dialogue available.");
                            }
                        }
                    }
                    None => {
                        add_text("! No NPC to talk to here.");
                    }
                }
            }
            &"say" => {
                match argument.as_str() {
                    "1" => {
                        add_text("You said: 'Nothing'");
                    }
                    "2" => {
                        add_text("You said: 'Gate key'");
                        add_text("You acquired a key");
                        
                    }
                    _ => {
                        add_text("#(FF0000)! Unknown argument for 'say' command.");
                    }
                }
            }
            &"save" => {
                let save_data = SaveData {
                    version: 1,
                    map_pack: crate::MAP_PACK.lock().unwrap().clone(),
                    player_name: crate::PLAYER_NAME.lock().unwrap().clone(),
                    last_save: chrono::Local::now().format("%Y-%m-%d %H:%M:%S").to_string(),
                    current_map_id: crate::CURRENT_MAP_ID.lock().unwrap().clone(),
                    health: *crate::PLAYER_HEALTH.lock().unwrap(),
                    stamina: *crate::PLAYER_STAMINA.lock().unwrap(),
                    coords: crate::PLAYER_COORDS.lock().unwrap().iter().map(|(k, v)| (k.to_string(), vec![v.0, v.1])).collect(),
                    inventory: crate::PLAYER_INVENTORY.lock().unwrap().clone(),
                    flags: crate::FLAGS.lock().unwrap().clone(),
                    lines_of_text: crate::LINES_OF_TEXT.lock().unwrap().clone(),
                };

                let result = save_json("save.json", to_value(save_data).unwrap());
                match result {
                    Ok(_) => {
                        add_text("#(00FF00)Progress saved");
                    }
                    Err(e) => {
                        add_text(format!("#(FF0000)Failed to save: {:?}", e).as_str());
                    }
                }
            }
            _ => {
                print!("{}", input.red());
            }
        }
        Some((synonym, argument))
    } else {
        print!("{}", input.red());
        None
    }
}

pub fn get_available_commands() {
    let width = if let Some((terminal_size::Width(w), _)) = terminal_size::terminal_size() {
        w
    } else {
        80 // Default height if terminal size cannot be determined
    };

    let events = get_map_events(get_current_map().unwrap().id.as_str()).unwrap();
    let all_coords = crate::PLAYER_COORDS.lock().unwrap().clone();
    let player_coords = all_coords.get(get_current_map().unwrap().id.as_str()).unwrap();
    let evt_item = events.iter().find(|e| e.evt_type == "item" && e.location[0] == player_coords.0 && e.location[1] == player_coords.1);
    let evt_location = events.iter().find(|e| e.evt_type == "location" && e.location[0] == player_coords.0 && e.location[1] == player_coords.1);
    let evt_npc = events.iter().find(|e| e.evt_type == "npc" && e.location[0] == player_coords.0 && e.location[1] == player_coords.1);
    let flags = crate::FLAGS.lock().unwrap();
    
    // GO
    move_cursor(width - 24, 4);
    print!("GO    | {} {} {} {}",
        if move_player(0, -1, true) { "▲".green() } else { "▲".red() },
        if move_player(0, 1, true) { "▼".green() } else { "▼".red() },
        if move_player(-1, 0, true) { "◀".green() } else { "◀".red() },
        if move_player(1, 0, true) { "▶".green() } else { "▶".red() }
    );

    // TAKE
    move_cursor(width - 24, 5);
    print!("TAKE  | {}", if evt_item.is_some() && !flags.contains_key(evt_item.unwrap().command_id.as_str())  { "✔".green() } else { "✘".red() });

    // ENTER/EXIT
    move_cursor(width - 24, 6);
    print!("ENTER | {}", if evt_location.is_some() { "✔".green() } else { "✘".red() });

    // TALK
    move_cursor(width - 24, 7);
    print!("TALK  | {}", if evt_npc.is_some() { "✔".green() } else { "✘".red() });
}
